package com.caimei.service.auth.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.caimei.mapper.cmMapper.AddressMapper;
import com.caimei.mapper.cmMapper.AuthMapper;
import com.caimei.model.ResponseJson;
import com.caimei.model.po.CmBrandAuthPo;
import com.caimei.model.po.TownPo;
import com.caimei.model.vo.AddressSelectVo;
import com.caimei.model.vo.RossAddressVo;
import com.caimei.module.base.entity.vo.TownVo;
import com.caimei.service.auth.AddressService;
import com.caimei.service.auth.UploadService;
import com.caimei.utils.RequestUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Description
 *
 * @author : Aslee
 * @date : 2022/1/6
 */
@Slf4j
@Service
public class AddressServiceImpl implements AddressService {
    @Resource
    private AddressMapper addressMapper;
    @Resource
    private AuthMapper authMapper;

    private UploadService uploadService;

    @Autowired
    public void setUploadService(UploadService uploadService) {
        this.uploadService = uploadService;
    }

    /**
     * 收货地址下拉选项列表
     *
     * @param type     选项类型：0省（默认），1市，2区
     * @param parentId 父级地址Id
     */
    @Override
    public ResponseJson<List<AddressSelectVo>> getSelectAddress(Integer type, Integer parentId) {
        if (0 == type) {
            // 获取所有省份列表
            List<AddressSelectVo> provinceList = addressMapper.getAllProvinceList();
            return ResponseJson.success(provinceList);
        } else if (1 == type) {
            // 获取当前省份下的所有城市列表
            List<AddressSelectVo> cityList = addressMapper.getCityListByProvinceId(parentId);
            return ResponseJson.success(cityList);
        } else if (2 == type) {
            // 获取当前城市下的所有地区列表
            List<AddressSelectVo> townList = addressMapper.getTownListByCityId(parentId);
            return ResponseJson.success(townList);
        }
        return ResponseJson.error("地址选项类型错误！", null);
    }

    @Override
    public ResponseJson<List<AddressSelectVo>> getAllSelectAddress() {
        // 获取所有省份列表
        List<AddressSelectVo> provinceList = addressMapper.getAllProvinceList();
        provinceList.forEach(province -> {
            // 获取当前省份下的所有城市列表
            List<AddressSelectVo> cityList = addressMapper.getCityListByProvinceId(province.getId());
            cityList.forEach(city -> {
                // 获取当前城市下的所有地区列表
                List<AddressSelectVo> townList = addressMapper.getTownListByCityId(city.getId());
                city.setChildren(townList);
            });
            province.setChildren(cityList);
        });
        return ResponseJson.success(provinceList);
    }

    @Override
    public ResponseJson addressTest() throws Exception {
        String ss ="0:\n" +
                "address:广东省广州市天河区珠江新城珠江东路6号周大福金融中心瑰丽府邸\"\n" +
                "allimg: \"/public/uploads/c6/f4db4f4167696ca54e7e9bd2f39918.jpg\"\n" +
                "description: \"圣贝拉\"\n" +
                "img: \"/public/uploads/c6/f4db4f4167696ca54e7e9bd2f39918.jpg\"\n" +
                "phone: \"4001061080\"\n" +
                "shop: \"广州分店\"\n" +
                "title: \"圣贝拉母婴护理中心\"\n" +
                "\n" +
                "1:\n" +
                "address: \"广东省广州市天河区-天河北路388号侨怡苑二层3号铺(天河北公交站)\"\n" +
                "allimg: \"/public/uploads/7c/e177aeb1b0178eddba92a5d6b85af1.jpg,/public/uploads/b0/05b0841950c69baf9603e9997669d7.jpg\"\n" +
                "description: \"颐天健\"\n" +
                "img: \"/public/uploads/3c/dd87c54e8ee48f8572d66af09b18ad.jpg\"\n" +
                "phone: \"1812742734\"\n" +
                "shop: \"广州-天河北店\"\n" +
                "title: \"颐天健中医养生连锁\"\n" +
                "\n" +
                "\n" +
                "2:\n" +
                "address: \"广东省广州市海珠区滨江东路555号金海湾二楼迪斯尼英语对面\"\n" +
                "allimg: \"\"\n" +
                "description: \"颐天健中医养生连锁\"\n" +
                "img: \"/public/uploads/3c/dd87c54e8ee48f8572d66af09b18ad.jpg\"\n" +
                "phone: \"4000933023\"\n" +
                "shop: \"广州-金海湾店\"\n" +
                "title: \"颐天健中医养生连锁\"\n" +
                "\n" ;

        String[] split = ss.split("\n");
        StringBuffer real = new StringBuffer();
        for (String s : split) {
            if (s.contains("address")) {
                real.append(s).append("\n");
            }
            if (s.contains("allimg")) {
                real.append(s).append("\n");
            }
            if (s.contains("description")) {
                real.append(s).append("\n");
            }
            if (s.contains("img") && !s.contains("allimg")) {
                real.append(s).append("\n");
            }
            if (s.contains("phone")) {
                real.append(s).append("\n");
            }
            if (s.contains("shop")) {
                real.append(s).append("\n");
            }
            if (s.contains("title")) {
                real.append(s).append("\n");
            }
        }
        String[] split1 = real.toString().split("\n");
        RossAddressVo address = new RossAddressVo();
        for (int i = 0; i < split1.length; i++) {
            if (0 == i % 7) {
                String[] split2 = split1[i].split(":");
                String s = split2[1];
                String s1 = cleanBlank(s);
                address.setAddress(s1);
            }
            if (1 == i % 7) {
                String[] split2 = split1[i].split(":");
                String s = split2[1];
                String s1 = cleanBlank(s);
                address.setAllimg(s1);
            }
            if (2 == i % 7) {
                String[] split2 = split1[i].split(":");
                String s = split2[1];
                String s1 = cleanBlank(s);
                address.setDescription(s1);
            }
            if (3 == i % 7) {
                String[] split2 = split1[i].split(":");
                String s = "http://spainross.com" + split2[1];
                String s1 = cleanBlank(s);
                address.setImgs(s1);
            }
            if (4 == i % 7) {
                String[] split2 = split1[i].split(":");
                String s = split2[1];
                String s1 = cleanBlank(s);
                address.setPhone(s1);
            }
            if (5 == i % 7) {
                String[] split2 = split1[i].split(":");
                String s = split2[1];
                String s1 = cleanBlank(s);
                address.setShopName(s1);
            }
            if (6 == i % 7) {
                String[] split2 = split1[i].split(":");
                String s = split2[1];
                String s1 = cleanBlank(s);
                address.setShopTitle(s1);
            }
            if (6 == i % 7) {
                CmBrandAuthPo cmBrandAuthPo = new CmBrandAuthPo();
                cmBrandAuthPo.setAuthParty(address.getShopTitle() + (StringUtils.isNotEmpty(address.getShopName()) ? "(" + address.getShopName() + ")" : ""));
                cmBrandAuthPo.setRemarks(address.getDescription());
                cmBrandAuthPo.setMobile(address.getPhone());
                cmBrandAuthPo.setAuthUserId(58);
                if (StringUtils.isNotEmpty(address.getImgs())) {
                    String imagePath = address.getImgs();
                    String fileName = imagePath.substring(imagePath.lastIndexOf("/") + 1);
                    String imageUrl = uploadService.saveFileByUrl(imagePath, fileName);
                    cmBrandAuthPo.setLogo(imageUrl);
                }
                cmBrandAuthPo.setLogo(address.getImgs());
                cmBrandAuthPo.setStatus(1);
                cmBrandAuthPo.setShopAuditStatus(1);
                cmBrandAuthPo.setAuditStatus(1);
                cmBrandAuthPo.setCustomFlag(0);
                cmBrandAuthPo.setFirstClubType(5);
                cmBrandAuthPo.setDelFlag(0);
                if (StringUtils.isNotBlank(address.getAddress())) {
                    String url = "https://restapi.amap.com/v3/geocode/geo?address="+address.getAddress()+"&key=9935007fc2048e62f9f593de49c346d9";
                    String result = RequestUtil.sendGet(url);
                    JSONObject jsonObject = JSONObject.parseObject(result);
                    String status = jsonObject.getString("status");
                    if ("1".equals(status)) {
                        String geocodes = jsonObject.getString("geocodes");
                        JSONArray jsonArray = JSONObject.parseArray(geocodes);
                        JSONObject addressInfo = JSONObject.parseObject(jsonArray.get(0).toString());
                        String lngAndLat = addressInfo.getString("location");
                        String[] locationSplit = lngAndLat.split(",");
                        BigDecimal lng = new BigDecimal(locationSplit[0]);
                        BigDecimal lat = new BigDecimal(locationSplit[1]);
                        cmBrandAuthPo.setLng(lng);
                        cmBrandAuthPo.setLat(lat);
                    }
                    Map<String, Object> town = getTown(address.getAddress());
                    TownPo to = (TownPo) town.get("town");
                    String str = (String) town.get("str");
                    cmBrandAuthPo.setAddress(str);
                    cmBrandAuthPo.setTownId(to.getTownId());
                    cmBrandAuthPo.setCityId(to.getCityId());
                    cmBrandAuthPo.setProvinceId(to.getProvinceId());
                }
                authMapper.insertAuth(cmBrandAuthPo);
                if (address.getAllimg().contains(",")) {
                    String[] split2 = address.getAllimg().split(",");
                    for (String s : split2) {
                        String imagePath = "http://spainross.com" + s;
                        String fileName = imagePath.substring(imagePath.lastIndexOf("/") + 1);
                        String imageUrl = uploadService.saveFileByUrl(imagePath, fileName);
                        authMapper.insertBanner(cmBrandAuthPo.getId(), imageUrl);
                    }
                } else {
                    String allimg = address.getAllimg();
                    String imagePath = "http://spainross.com" + allimg;
                    String fileName = imagePath.substring(imagePath.lastIndexOf("/") + 1);
                    String imageUrl = uploadService.saveFileByUrl(imagePath, fileName);
                    authMapper.insertBanner(cmBrandAuthPo.getId(), imageUrl);
                }
            }
        }
        return ResponseJson.success();
    }


    public String cleanBlank(String str) {
        if (str != null && (str.contains("\\") || str.contains("") || str.contains("\"") || str.contains(" \""))) {
            String replaceFlag = str.replace("\\", "");
            str = replaceFlag.replace("\"", "").replaceAll(" ", "").trim();
            return str;
        }
        return str;
    }

    public Map<String, Object> getTown(String str) {
        TownPo townPo = new TownPo();
        List<String> townNames = authMapper.getTownNames();
        for (String townName : townNames) {
            if (str.contains(townName)) {
                townPo = authMapper.getTownList(townName).get(0);
            }
            if(null!=townPo.getCityId()){
                townPo.setProvinceId(authMapper.getProvinceIdByCityId(townPo.getCityId()));
            }
        }
        if (null == townPo.getTownId()) {
            List<String> provinceNames = authMapper.getProvinceNames();
            for (String provinceName : provinceNames) {
                if (str.contains(provinceName)) {
                    townPo.setProvinceId(authMapper.getProvinceId(provinceName));
                }
            }
            List<String> cityNames = authMapper.getCityNames(townPo.getProvinceId());
            for (String cityName : cityNames) {
                if (str.contains(cityName)) {
                    townPo.setCityId(authMapper.getCityIdByProvinceId(townPo.getProvinceId(), cityName));
                }
            }
        }

        Integer provinceId = townPo.getProvinceId();
        if (null != provinceId) {
            String provinceName = authMapper.getProvinceNameById(provinceId);
            str = str.replace(provinceName, "");
        }
        Integer cityId = townPo.getCityId();
        if (null != cityId) {
            String cityName = authMapper.getCityNameByCityId(cityId);
            str = str.replace(cityName, "");
        }
        Integer townId = townPo.getTownId();
        if (null != townId) {
            String townName = authMapper.getTownNameById(townId);
            str = str.replace(townName, "");
        }
        String s = cleanBlank(str);
        HashMap<String, Object> map = new HashMap<>();
        map.put("town", townPo);
        map.put("str", s);
        return map;
    }
}
