package com.caimei.service.auth.impl;

import com.alibaba.fastjson.JSONArray;
import com.caimei.config.FastDfsClient;
import com.caimei.mapper.cmMapper.AuthMapper;
import com.caimei.mapper.cmMapper.AuthProductMapper;
import com.caimei.mapper.cmMapper.LogisticsMapper;
import com.caimei.model.ResponseJson;
import com.caimei.model.dto.ProductSaveDto;
import com.caimei.model.po.*;
import com.caimei.model.vo.*;
import com.caimei.service.auth.*;
import com.caimei.utils.ExcelOperateUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.github.tobato.fastdfs.service.FastFileStorageClient;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.usermodel.*;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.usermodel.*;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.imageio.ImageIO;
import javax.servlet.http.HttpServletResponse;
import java.awt.image.BufferedImage;
import java.io.*;
import java.math.BigDecimal;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * Description
 *
 * @author : Aslee
 * @date : 2021/5/11
 */
@Slf4j
@Service
public class LogisticsServiceImpl implements LogisticsService {

    @Resource
    private LogisticsMapper logisticsMapper;

    @Resource
    private AuthMapper authMapper;

    @Value("${spring.profiles.active}")
    private String active;

    @Value("${caimei.imageDomain}")
    private String imageDomain;

    @Override
    public ResponseJson sendCertificate(Integer authId, Integer companyId, String logisticsNumber, JSONArray imageList, String remarks) {
        List<LogisticsCompanyVo> logisticsCompanyList = logisticsMapper.getLogisticsCompanyList(companyId);
        if (null == logisticsCompanyList || logisticsCompanyList.size() == 0) {
            return ResponseJson.error("物流公司不存在");
        }
        LogisticsCompanyVo logisticsCompany = logisticsCompanyList.get(0);
        CertificateLogisticsPo logistics = new CertificateLogisticsPo();
        logistics.setAuthId(authId);
        logistics.setCompanyName(logisticsCompany.getCompanyName());
        logistics.setCompanyCode(logisticsCompany.getCompanyCode());
        logistics.setLogisticsNumber(logisticsNumber);
        logistics.setRemarks(remarks);
        String imageRemarks = "";
        if (null != imageList){
            String[] imagePath = new String[imageList.size()];
            for (int i = 0; i < imagePath.length; i++) {
                imagePath[i] = imageList.get(i).toString();
            }
            imageRemarks = StringUtils.join(imagePath, "##");
        }
        logistics.setRemarksImage(imageRemarks);
        logistics.setType(4);
        logistics.setState(1);
        logistics.setUpdateDate(new Date());
        logisticsMapper.sendCertificate(logistics);
        // 更改机构寄送状态
        authMapper.updateSendStatus(authId);
        return ResponseJson.success("寄送成功");
    }

    @Override
    public ResponseJson certificateLogistics(Integer authId) {
        CertificateLogisticsVo certificateLogistics = logisticsMapper.getCertificateLogistics(authId);
        String[] imageArr = certificateLogistics.getRemarksImage().split("##");
        List<String> imageList = Arrays.asList(imageArr);
        certificateLogistics.setImageList(imageList);
        List<RouterVo> routers = JSONArray.parseArray(certificateLogistics.getInfo(), RouterVo.class);
        certificateLogistics.setRouterList(routers);
        return ResponseJson.success(certificateLogistics);
    }

    @Override
    public ResponseJson getLogisticsCompany() {
        List<LogisticsCompanyVo> logisticsCompanyList = logisticsMapper.getLogisticsCompanyList(null);
        return ResponseJson.success(logisticsCompanyList);
    }
}
